#pragma once

#include "Volume.h"
#include "Point3d.h"
#include <vector>


using namespace std;



class StreamlineRenderer								//Traces and renders 3D streamlines in a 3D uniform vector field
{														//represented as a Volume<Point3d>.
public:

				StreamlineRenderer();
			   ~StreamlineRenderer();

void			setSeeds(const vector<Point3d>& s)
				{ seeds = s; }

void			setMaxLength(float l)
				{ max_length = l; }			

float			getMaxLength()
				{ return max_length; }			

void			setStepSize(float s)
				{ step_size = s; }			

float			getStepSize()
				{ return step_size; }
				
void			setThickness(float t)
				{ thickness = t; }			

float			getThickness()
				{ return thickness; }			
				
void			setMinLength(float l)
				{ if (l>=0) min_length = l; }

float			getMinLength()
				{ return min_length; }

void			computeStreamlines(Volume<Point3d>&);	//Compute all streamlines for the current setting, and save them.
														//This makes next the draw() function go much faster.

void			draw(const Volume<float>& scalar);		//Draw the already-computed streamlines, with transparency modulated
														//by the scalar field 'scalar'.

protected:

struct StreamPoint										//A point along a computed streamline. 
{
				StreamPoint() {}
Point3d			point;									//Point position
Point3d			vector;									//Vector value at current point
float			magnitude;								//Vector magnitude at current point
float			position;								//Position of point along streamline (in [0,1] arc-length coordinates)
};

struct StreamLine										//A streamline is a collection of stream points, and some extra data
{														//such as the streamline length
vector<StreamPoint> points;				
float				length;
};

float			max_length;								//Maximum allowed streamline length
float			min_length;								//Minimum streamline length for streamlines to be drawn
float			step_size;								
float			thickness;
float			vector_min,vector_max;
vector<Point3d> seeds;
vector<StreamLine*> 
				streamlines;

void			trace(Volume<Point3d>& g,const Point3d& seed,StreamLine& streamline);				
float           traceOneDirection(bool downstream,Volume<Point3d>& g,const Point3d& seed,StreamLine& streamline);

private:

void			computeVertex(const Volume<Point3d>& g,int i,int j,int k,float* vertex, float* result);
bool			getC1Vector(const Volume<Point3d>& g,const float* position, float* value);
void			draw(const StreamLine& streamline,const Volume<float>& scalar);
};




						

